import { Bytes, fromJS } from '../src/values';
import './test-utils';

describe('immutable byte arrays', () => {
    describe('Uint8Array methods', () => {
        const bs = Bytes.of(10, 20, 30, 40);
        it('should yield entries', () => {
            expect(fromJS(Array.from(bs.entries())))
                .is(fromJS([[0,10],[1,20],[2,30],[3,40]]));
        });
        it('should implement every', () => {
            expect(bs.every((b) => !(b & 1))).toBe(true);
            expect(bs.every((b) => b !== 50)).toBe(true);
            expect(!(bs.every((b) => b !== 20))).toBe(true);
        });
        it('should implement find', () => {
            expect(bs.find((b) => b > 20)).toBe(30);
            expect(bs.find((b) => b > 50)).toBe(void 0);
        });
        it('should implement findIndex', () => {
            expect(bs.findIndex((b) => b > 20)).toBe(2);
            expect(bs.findIndex((b) => b > 50)).toBe(-1);
        });
        it('should implement forEach', () => {
            const vs: number[] = [];
            bs.forEach((b) => vs.push(b));
            expect(fromJS(vs)).is(fromJS([10, 20, 30, 40]));
        });
        it('should implement includes', () => {
            expect(bs.includes(20)).toBe(true);
            expect(!bs.includes(50)).toBe(true);
        });
        it('should implement indexOf', () => {
            expect(bs.indexOf(20)).toBe(1);
            expect(bs.indexOf(50)).toBe(-1);
        });
        it('should implement join', () => {
            expect(bs.join('-')).toBe('10-20-30-40');
        });
        it('should implement keys', () => {
            expect(fromJS(Array.from(bs.keys()))).is(fromJS([0,1,2,3]));
        });
        it('should implement values', () => {
            expect(fromJS(Array.from(bs.values()))).is(fromJS([10,20,30,40]));
        });
        it('should implement filter', () => {
            expect(bs.filter((b) => b !== 30)).is(Bytes.of(10,20,40));
        });
        it('should implement slice', () => {
            const vs = bs.slice(2);
            expect(Object.is(vs._view.buffer, bs._view.buffer)).toBe(false);
            expect(vs._view.buffer.byteLength).toBe(2);
            expect(vs.get(0)).toBe(30);
            expect(vs.get(1)).toBe(40);
            expect(vs.length).toBe(2);
        });
        it('should implement subarray', () => {
            const vs = bs.subarray(2);
            expect(Object.is(vs._view.buffer, bs._view.buffer)).toBe(true);
            expect(vs._view.buffer.byteLength).toBe(4);
            expect(vs.get(0)).toBe(30);
            expect(vs.get(1)).toBe(40);
            expect(vs.length).toBe(2);
        });
        it('should implement reverse', () => {
            const vs = bs.reverse();
            expect(Object.is(vs._view.buffer, bs._view.buffer)).toBe(false);
            expect(bs.get(0)).toBe(10);
            expect(bs.get(3)).toBe(40);
            expect(vs.get(0)).toBe(40);
            expect(vs.get(3)).toBe(10);
        });
        it('should implement sort', () => {
            const vs = bs.reverse().sort();
            expect(Object.is(vs._view.buffer, bs._view.buffer)).toBe(false);
            expect(bs.get(0)).toBe(10);
            expect(bs.get(3)).toBe(40);
            expect(vs.get(0)).toBe(10);
            expect(vs.get(3)).toBe(40);
        });
    });
});
